import os
import yaml
import glob
from pathlib import Path
import subprocess


def update_config(input_folder, output_folder, recursive):
    """
    Updates the `config.yaml` file with the provided input and output folders.
    """
    # Resolve the path to the `config.yaml` dynamically
    library_path = Path(__file__).resolve().parent
    config_path = library_path / "config/config.yaml"
    reference_path = library_path / "config/NC_004296.fasta"
    model_path = library_path / "config/RF_LASV_lineage_n1000_aa.joblib"
    
    print(f"Config path resolved to: {config_path}")

    # Ensure the config directory exists
    config_path.parent.mkdir(parents=True, exist_ok=True)

    # Read or create the configuration
    if config_path.exists():
        with config_path.open("r") as file:
            config = yaml.safe_load(file) or {}
    else:
        print(f"Warning: Config file {config_path} does not exist. A new one will be created.")
        config = {}

    # Update the configuration
    config["raw_seq_folder"] = str(input_folder)
    config["output"] = str(output_folder)
    config["reference"] = str(reference_path)
    config["filter"] = {"min_length": 500}
    config["model"] = str(model_path)
    config["figures_title"] = "LASV Lineage Prediction"
    config["recursive"] = True if recursive else False

    # Save the updated configuration
    with config_path.open("w") as config_file:
        yaml.dump(config, config_file)

    print(f"Config updated successfully at {config_path}")


def run_pipeline(input_folder, output_folder, recursive, cores, force):
    """
    Runs the Snakemake pipeline with the specified parameters.
    """
    # Resolve paths for input and output
    input_path = Path(input_folder).resolve()
    output_path = Path(output_folder).resolve()

    # Resolve library paths
    library_path = Path(__file__).resolve().parent
    snakefile_path = library_path / "predict_lineage.smk"
    config_path = library_path / "config/config.yaml"

    # Validate paths
    if not input_path.exists():
        print(f"Error: Input folder '{input_path}' does not exist.")
        exit(1)

    if not snakefile_path.exists():
        print(f"Error: Snakefile '{snakefile_path}' does not exist.")
        exit(1)

    # Create output directory if it doesn't exist
    output_path.mkdir(parents=True, exist_ok=True)

    # Update configuration and collect FASTA files
    update_config(input_path, output_path, recursive)
    
    # Construct the Snakemake command
    snakemake_command = [
        "snakemake",
        "-s", str(snakefile_path),
        "--configfile", str(config_path),
        "--cores", str(cores),
        "--rerun-incomplete", #always rerun incomplete files
    ]
    if force:
        snakemake_command.append("--forceall")

    # Run Snakemake
    print(f"Running Snakemake with command: {' '.join(snakemake_command)}")
    try:
        subprocess.run(snakemake_command, check=True)
    except subprocess.CalledProcessError as e:
        print(f"Error running Snakemake: {e}")
        exit(1)



from setuptools import setup, find_packages
from setuptools.command.install import install
import subprocess
import os
import sys

# Custom post-install step to install Nextclade
def install_nextclade():
    try:
        print("Installing Nextclade...")
        subprocess.run(["python", "-m", "CLASV.install_nextclade"], check=True)
        print("Nextclade installation completed.")
        print(
            "\nIMPORTANT: To ensure the Nextclade CLI is available, you may need to restart your terminal "
            "or run the following command:\n"
        )
    except Exception as e:
        print(f"Failed to install Nextclade: {e}")

class CustomInstallCommand(install):
    def run(self):
        install.run(self)
        install_nextclade()


print('Running setup...')

setup(
    name='CLASV',
    version='0.1.14',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
            "appdirs==1.4.4",
        "argparse-dataclass==2.0.0",
        "attrs==24.2.0",
        "biopython==1.84",
        "certifi==2024.8.30",
        "charset-normalizer==3.4.0",
        "conda-inject==1.3.2",
        "ConfigArgParse==1.7",
        "connection_pool==0.0.3",
        "contourpy==1.3.1",
        "cycler==0.12.1",
        "datrie==0.8.2",
        "docutils==0.21.2",
        "dpath==2.2.0",
        "fastjsonschema==2.21.1",
        "fonttools==4.55.2",
        "gitdb==4.0.11",
        "GitPython==3.1.43",
        "humanfriendly==10.0",
        "idna==3.10",
        "immutables==0.21",
        "Jinja2==3.1.4",
        "joblib==1.4.2",
        "jsonschema==4.23.0",
        "jsonschema-specifications==2024.10.1",
        "jupyter_core==5.7.2",
        "kiwisolver==1.4.7",
        "MarkupSafe==3.0.2",
        "matplotlib==3.9.3",
        "nbformat==5.10.4",
        "numpy==1.23.5",
        "packaging==24.2",
        "pandas==2.2.3",
        "pillow==11.0.0",
        "plac==1.4.3",
        "platformdirs==4.3.6",
        "plotly==5.24.1",
        "psrecord==1.4",
        "psutil==6.1.0",
        "PuLP==2.9.0",
        "pyparsing==3.2.0",
        "python-dateutil==2.9.0.post0",
        "pytz==2024.2",
        "PyYAML==6.0.2",
        "referencing==0.35.1",
        "requests==2.32.3",
        "reretry==0.11.8",
        "rpds-py==0.22.3",
        "scikit-learn==1.2.2",
        "scipy==1.14.1",
        "six==1.17.0",
        "smart-open==7.0.5",
        "smmap==5.0.1",
        "snakemake==8.25.5",
        "snakemake-interface-common==1.17.4",
        "snakemake-interface-executor-plugins==9.3.2",
        "snakemake-interface-report-plugins==1.1.0",
        "snakemake-interface-storage-plugins==3.3.0",
        "tabulate==0.9.0",
        "tenacity==9.0.0",
        "threadpoolctl==3.5.0",
        "throttler==1.2.2",
        "traitlets==5.14.3",
        "tzdata==2024.2",
        "urllib3==2.2.3",
        "wrapt==1.17.0",
        "yte==1.5.4",
        "zipp==3.21.0"
    ],
    package_data={
        "CLASV": [
            "predict_lineage.smk", 
            "config/config.yaml",
            "*.smk",
            "config/*.yaml",
            "results/*.fasta",
            "results/*.csv",
            "predictions/*.csv",
            "visuals/*.html"
        ]
    },
    entry_points={
        "console_scripts": [
            "clasv=CLASV.cli:main",
        ]
    },
    description='CLASV is a pipeline designed for rapidly predicting Lassa virus lineages using a Random Forest model.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Richard Daodu, Ebenezer Awotoro',
    author_email='lordrichado@gmail.com',
    url='https://github.com/JoiRichi/CLASV/commits?author=JoiRichi',
    python_requires=">=3.6"
,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    cmdclass={
        "install": CustomInstallCommand,
    },
)



import argparse
import subprocess
from CLASV.pipeline import run_pipeline
from CLASV.install_nextclade import install_nextclade, is_nextclade_installed
    
    
def main():
    parser = argparse.ArgumentParser(prog="clasv", description="CLASV: Lassa Virus Analysis Pipeline")
    subparsers = parser.add_subparsers(dest="command", help="Available subcommands")

    # Subcommand: find-lassa
    find_parser = subparsers.add_parser("find-lassa", help="Find Lassa virus sequences")
    find_parser.add_argument("--input", required=True, help="Path to the input folder.")
    find_parser.add_argument("--output", required=True, help="Path to the output folder.")
    find_parser.add_argument("--recursive", action="store_true", help="Search input folder recursively.")
    find_parser.add_argument("--cores", type=int, default=4, help="Number of cores to use (default: 4).")
    find_parser.add_argument("--force", action="store_true", help="Force rerun of all pipeline steps.")

    args = parser.parse_args()

    # Handle subcommands
    if args.command == "find-lassa":
        if is_nextclade_installed():
            print("Nextclade installation verified successfully.")
            run_pipeline(args.input, args.output, args.recursive, args.cores, args.force)
        else:
            print("Nextclade has not been installed. Installation in progress. If Nextclade installs and the analysis did not auto-continue, please rerun your command.")
            install_nextclade()
            run_pipeline(args.input, args.output, args.recursive, args.cores, args.force)
            
    else:
        parser.print_help()
